"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransactionProvider = void 0;
// import isString from 'lodash/isString'
// import isNumber from 'lodash/isNumber'
var bignumber_js_1 = require("bignumber.js");
var enum_1 = require("../types/enum");
var Bytes = require('eth-lib/lib/bytes');
console.log(Bytes.toArray('0xc87b56dd0000000000000000000000000000000000000000000000000000000000012aa7'));
console.log(Bytes.fromArray(Bytes.toArray('0xc87b56dd0000000000000000000000000000000000000000000000000000000000012aa7')));
var RLP = require('eth-lib/lib/rlp');
// const Hash = require('eth-lib/lib/hash')
var encodeWei = function (wei) {
    if (wei === null || wei === undefined) {
        return Bytes.fromNat('0x0');
    }
    else if (wei.isEqualTo(new bignumber_js_1.default(0))) {
        return Bytes.fromNat('0x0');
    }
    else {
        return Bytes.fromNumber(wei);
    }
};
var TransactionProvider = /** @class */ (function () {
    function TransactionProvider() {
    }
    TransactionProvider.prototype.signAndSerializeTx = function () { };
    TransactionProvider.prototype.serializeTx = function (tx) {
        var encodedTxType = RLP.encode(Bytes.fromNumber(tx.type));
        var rplInput = [];
        switch (tx.type) {
            case enum_1.THETA_TRANSACTION_TYPE_ENUM.smart_contract:
                var valueWeiBN = bignumber_js_1.default.isBigNumber(tx.value) ? tx.value : new bignumber_js_1.default(tx.value);
                var ten18 = new bignumber_js_1.default(10).pow(18); // 10^18, 1 Theta = 10^18 ThetaWei, 1 Gamma = 10^ TFuelWei
                var feeInTFuelWei = new bignumber_js_1.default(tx.gasPrice).multipliedBy(ten18); //
                var from = new TxInput(tx.fromAddress, null, valueWeiBN, tx.senderSequence);
                rplInput.push(from.rlpInput());
                var to = new TxOutput(tx.toAddress, null, null);
                rplInput.push(to.rlpInput());
                rplInput.push(Bytes.fromNumber(tx.gasLimit));
                rplInput.push(encodeWei(feeInTFuelWei));
                rplInput.push(tx.data);
                break;
            default:
                break;
        }
        var encodedTx = RLP.encode(rplInput); // this time encode with signature
        var signedRawBytes = encodedTxType + encodedTx.slice(2);
        return signedRawBytes;
    };
    TransactionProvider.prototype.signTx = function () { };
    return TransactionProvider;
}());
exports.TransactionProvider = TransactionProvider;
var TxInput = /** @class */ (function () {
    function TxInput(address, thetaWei, tfuelWei, sequence) {
        this.address = address;
        this.sequence = sequence;
        this.signature = '';
        if (thetaWei || tfuelWei) {
            this.coins = new Coins(thetaWei, tfuelWei);
        }
        else {
            //TODO should this be undefined or null?
            this.coins = new Coins(null, null);
        }
    }
    TxInput.prototype.setSignature = function (signature) {
        this.signature = signature;
    };
    TxInput.prototype.rlpInput = function () {
        var address = null;
        if (this.address) {
            address = this.address.toLowerCase();
        }
        else {
            address = Bytes.fromNat('0x0');
        }
        var rplInput = [address, this.coins.rlpInput(), Bytes.fromNumber(this.sequence), this.signature];
        return rplInput;
    };
    return TxInput;
}());
var TxOutput = /** @class */ (function () {
    function TxOutput(address, thetaWei, tfuelWei) {
        this.address = address;
        if (thetaWei || tfuelWei) {
            this.coins = new Coins(thetaWei, tfuelWei);
        }
        else {
            //TODO should this be undefined or null?
            this.coins = new Coins(null, null);
        }
    }
    TxOutput.prototype.rlpInput = function () {
        var address = null;
        if (this.address) {
            address = this.address.toLowerCase();
        }
        else {
            //Empty address
            address = '0x0000000000000000000000000000000000000000';
        }
        var rplInput = [address, this.coins.rlpInput()];
        return rplInput;
    };
    return TxOutput;
}());
var Coins = /** @class */ (function () {
    function Coins(thetaWei, tfuelWei) {
        this.thetaWei = thetaWei;
        this.tfuelWei = tfuelWei;
    }
    Coins.prototype.rlpInput = function () {
        var rlpInput = [
            encodeWei(this.thetaWei),
            encodeWei(this.tfuelWei)
            //(this.thetaWei.isEqualTo(new BigNumber(0))) ? Bytes.fromNat("0x0") : Bytes.fromNumber(this.thetaWei),
            //(this.tfuelWei.isEqualTo(new BigNumber(0))) ? Bytes.fromNat("0x0") : Bytes.fromNumber(this.tfuelWei)
        ];
        return rlpInput;
    };
    return Coins;
}());
//# sourceMappingURL=TransactionProvider.js.map