import { THETA_BLOCK_STATUS_ENUM, THETA_TRANSACTION_TYPE_ENUM } from './enum';
import { CmcHttpProvider } from '../providers/CmcHttpProvider';
import { ThetaHttpProvider } from '../providers/ThetaHttpProvider';
export interface THETA_NETWORK_INTERFACE {
    chainId: string;
    name?: string;
    rpcUrl: string;
    explorerUrl: string;
}
export interface THETA_GET_BLOCK_INTERFACE {
    jsonrpc: '2.0';
    id: number;
    result: THETA_BLOCK_INTERFACE;
}
export interface THETA_GET_BLOCKS_BY_RANGE_INTERFACE {
    jsonrpc: '2.0';
    id: number;
    result: Array<THETA_BLOCK_INTERFACE>;
}
export interface THETA_BLOCK_INTERFACE {
    chain_id: 'mainnet' | 'testnet';
    epoch: string;
    height: string;
    parent: string;
    transactions_hash: string;
    state_hash: string;
    timestamp: string;
    proposer: string;
    hcc: {
        Votes: Array<{
            Block: string;
            Height: number;
            Epoch: number;
            ID: string;
            Signature: string;
        }>;
        BlockHash: string;
    };
    guardian_votes: null | {
        Block: string;
        Gcp: string;
        Multiplies: Array<number>;
    };
    elite_edge_node_votes: null | {
        Block: string;
        Multiplies: Array<number>;
        Addresses: Array<string>;
    };
    children: Array<string>;
    status: THETA_BLOCK_STATUS_ENUM;
    hash: string;
    transactions: Array<{
        raw: {
            gas_limit?: string;
            gas_price?: string;
            proposer: {
                address: string;
                coins: {
                    thetawei: string;
                    tfuelwei: string;
                };
                sequence: string;
                signature: string;
            };
            fee?: {
                thetawei: string;
                tfuelwei: string;
            };
            inputs?: Array<{
                address: string;
                coins: {
                    thetawei: string;
                    tfuelwei: string;
                };
                sequence: string;
                signature: string;
            }>;
            outputs?: Array<{
                address: string;
                coins: {
                    thetawei: string;
                    tfuelwei: string;
                };
            }>;
            block_height: string;
            holder?: {
                address: string;
                coins: {
                    thetawei: string;
                    tfuelwei: string;
                };
            };
            source?: {
                address: string;
                coins: {
                    thetawei: string;
                    tfuelwei: string;
                };
                sequence: string;
                signature: string;
            };
            purpose?: number;
            data?: string;
            from?: {
                address: string;
                coins: {
                    thetawei: string;
                    tfuelwei: string;
                };
                sequence: string;
                signature: string;
            };
            to?: {
                address: string;
                coins: {
                    thetawei: string;
                    tfuelwei: string;
                };
                sequence: string;
                signature: string;
            };
        };
        type: THETA_TRANSACTION_TYPE_ENUM;
        hash: string;
        receipt?: {
            TxHash: string;
            Logs: Array<{
                address: string;
                topics: Array<string>;
                data: string;
            }>;
            EvmRet: string;
            ContractAddress: string;
            GasUsed: number;
            EvmErr: string;
        };
    }>;
}
export interface HTTP_PROVIDER_INTERFACE {
    getBlock(blockHeight: number): Promise<THETA_BLOCK_INTERFACE>;
    getBlock(blockHash: string): Promise<THETA_BLOCK_INTERFACE>;
}
export interface THETA_METHOD_INTERFACE {
    method: 'theta.GetAccount' | 'theta.BroadcastRawTransaction';
}
export interface THETA_VCP_INTERFACE {
    jsonrpc: '2.0';
    id: number;
    result: {
        BlockHashVcpPairs: [
            {
                BlockHash: string;
                Vcp: {
                    SortedCandidates: Array<{
                        Holder: string;
                        Stakes: Array<{
                            source: string;
                            amount: string;
                            withdrawn: boolean;
                            return_height: string;
                        }>;
                    }>;
                };
                HeightList: {
                    Heights: Array<number>;
                };
            }
        ];
    };
}
export interface THETA_GCP_INTERFACE {
    jsonrpc: string;
    id: number;
    result: {
        BlockHashGcpPairs: [
            {
                BlockHash: string;
                Gcp: {
                    SortedGuardians: Array<{
                        Holder: string;
                        Stakes: Array<{
                            source: string;
                            amount: string;
                            withdrawn: boolean;
                            return_height: string;
                        }>;
                    }>;
                };
                HeightList: {
                    Heights: Array<number>;
                };
            }
        ];
    };
}
export interface THETA_EENP_INTERFACE {
    jsonrpc: string;
    id: number;
    result: {
        BlockHashEenpPairs: [
            {
                BlockHash: string;
                EENs: Array<{
                    Holder: string;
                    Stakes: Array<{
                        source: string;
                        amount: string;
                        withdrawn: boolean;
                        return_height: string;
                    }>;
                }>;
                HeightList: {
                    Heights: Array<number>;
                };
            }
        ];
    };
}
export interface THETA_NODE_STATUS {
    jsonrpc: '2.0';
    id: 1;
    result: {
        address: string;
        chain_id: string;
        peer_id: string;
        latest_finalized_block_hash: string;
        latest_finalized_block_height: string;
        latest_finalized_block_time: string;
        latest_finalized_block_epoch: string;
        current_epoch: string;
        current_height: string;
        current_time: string;
        syncing: false;
        genesis_block_hash: '0xd8836c6cf3c3ccea0b015b4ed0f9efb0ffe6254db793a515843c9d0f68cbab65';
    };
}
export interface THETA_GET_VERSION_INTERFACE {
    jsonrpc: string;
    id: number;
    result: {
        version: string;
        git_hash: string;
        timestamp: string;
    };
}
export interface THETA_GET_ACCOUNT_INTERFACE {
    jsonrpc: string;
    id: number;
    result: {
        sequence: number;
        coins: {
            thetawei: string;
            tfuelwei: string;
        };
        reserved_funds: Array<number>;
        last_updated_block_height: string;
        root: string;
        code: string;
    };
}
export interface THETA_TRANSACTION_INTERFACE {
    jsonrpc: string;
    id: number;
    result: {
        block_hash: string;
        block_height: string;
        status: string;
        hash: string;
        transaction: {
            fee: {
                thetawei: string;
                tfuelwei: string;
            };
            inputs: Array<{
                address: string;
                coins: {
                    thetawei: string;
                    tfuelwei: string;
                };
                sequence: string;
                signature: string;
            }>;
            outputs: Array<{
                address: string;
                coins: {
                    thetawei: string;
                    tfuelwei: string;
                };
            }>;
        };
    };
}
export interface THETA_GET_PENDING_TRANSACTIONS_INTERFACE {
    jsonrpc: string;
    id: number;
    result: {
        tx_hashes: Array<string>;
    };
}
export interface THETA_GET_STAKE_REWARD_DISTRIBUTION_BY_HEIGHT_INTERFACE {
    jsonrpc: string;
    id: number;
    result: {
        BlockHashStakeRewardDistributionRuleSetPairs: Array<{
            BlockHash: string;
            StakeRewardDistributionRuleSet: Array<{
                Beneficiary: string;
                SplitBasisPoint: string;
                StakeHolder: string;
            }>;
        }>;
    };
}
export interface THETA_CALL_SMART_CONTRACT_INTERFACE {
    jsonrpc: string;
    id: number;
    result: {
        vm_return: string;
        contract_address: string;
        gas_used: string;
        vm_error: string;
    };
}
export interface CMC_PRICE_INFORMATION {
    name: string;
    price: number;
    volume_24h: number;
    market_cap: number;
    total_supply: number;
    circulating_supply: number;
    last_updated: string;
}
export interface THETA_TS_SDK {
    cmc: CmcHttpProvider;
    blockchain: ThetaHttpProvider;
}
